@@ -1,5 +1,5 @@
-from __future__ import division, print_function
-from visual import *
+#
+from vpython import *
 
 G = 6.7e-11
 
